library("LOLA")
library("GenomicRanges")

# PATHS
regionDB = loadRegionDB("LOLAcore/nm/t1/resources/regions/LOLACore/hg38/")
iterate_over <- dir("significant_bins", pattern ="_significant_FDR_shorterfrag_bins.bed",full.names = TRUE)


for(i in 1:length(iterate_over)){
    if (file.info(iterate_over[i])$size != 0) {
        
        # Prepare input
        name=strsplit(iterate_over[i],"/")[[1]][2]
        samplename <- gsub("_significant_FDR_shorterfrag_bins.bed","",c(paste0(c(name,""),collapse = "")))
        print("working on",samplename)
        
        nonCNV_bins_bed=paste0(c("../",samplename,"_nonCNVaffected_bins.bed"),collapse = "") # used as universe
        shorter_bed=paste0(c("../",samplename,"_significant_FDR_shorterfrag_bins.bed"),collapse = "") # used as main input
        longer_bed=paste0(c("../",samplename,"_significant_FDR_longerfrag_bins.bed"),collapse = "") # used as main input
        random_shorter_bed=paste0(c("../",samplename,"_random_nonCNVaffected_bins_as_many_as_sign_shorter.bed"),collapse = "") # used as control input
        random_longer_bed=paste0(c("../",samplename,"_random_nonCNVaffected_bins_as_many_as_sign_longer.bed"),collapse = "") # used as control input

        universe <- readBed(nonCNV_bins_bed)
        testsetnames=c("shorter","longer","random_shorter","random_longer","middle_40_to_60_")
        j <- 0
        for(testset in c(shorter_bed,longer_bed,random_shorter_bed,random_longer_bed,middle_40_to_60_bed)){
            print(testset)
            j <- j+1
            if ((file.exists(testset)) & (file.info(testset)$size != 0)) {

                userSet= GRangesList(c(readBed(testset)))

                names(userSet) <- c(samplename)
                # actual LOLA:
                locResults = runLOLA(userSet, universe, regionDB, cores=10)
                outdirname=paste0(c("lolaResults_",testsetnames[j]),collapse = "")
                writeCombinedEnrichment(locResults, outFolder=outdirname, includeSplits=TRUE)
                system(paste0(c("mv ",outdirname,"/allEnrichments.tsv ",outdirname,"/",samplename,".tsv"),collapse = ""))
            }
        }
    }
}