#!/usr/bin/env python

import pandas as pd
import matplotlib
#matplotlib.use('Agg')
import matplotlib.pyplot as plt
#matplotlib.rcParams.update({'font.size': 14})
import scipy.optimize
import numpy as np
from numpy import exp
from lmfit.models import GaussianModel, ConstantModel
import glob
import scipy
import subprocess
import sys
from pathlib import Path
import lmfit
import math

################## PARAMS ####################

corr_cov_glob=sys.argv[1]
extdto=int(sys.argv[2])
bs=int(sys.argv[3])
avg_centersize=int(sys.argv[4])

method="leastsq"#""  # default is "leastsq" (Levenberg-Marquardt)
print_sigmas=False

try:
    g1_min_sigma=float(sys.argv[5])
    g1_max_sigma=float(sys.argv[6])
    g2_min_sigma=float(sys.argv[7])
    g2_max_sigma=float(sys.argv[8])
    g3_min_sigma=float(sys.argv[9])
    g3_max_sigma=float(sys.argv[10])

except:
    print_sigmas=True
    g1_min_sigma=20
    g1_max_sigma=200
    g2_min_sigma=200
    g2_max_sigma=3000
    g3_min_sigma=3000
    g3_max_sigma=40000

    
corr_cov_filelist=glob.glob(corr_cov_glob)
outdir="fitted_gaussians_sigmas_%dto%d_%dto%d_%dto%d"%(g1_min_sigma,g1_max_sigma,g2_min_sigma,g2_max_sigma,g3_min_sigma,
                                                   g3_max_sigma)
################ CODE ###########################


subprocess.check_call("mkdir -p %s"%(outdir,),shell=True)

paramlist=[]

for covfile in corr_cov_filelist:

    samplename=str(Path(covfile).parent).split("/")[-1]
    sampledict={"sample":samplename}
    print("Running on %s"%(samplename))
    df=pd.read_csv(covfile)

    central_bin_x_values=[x*avg_centersize-avg_centersize/2 for x in [0.05,0.175,0.5,0.825,0.95]]

    df=df[["bin","averagecoverage"]]
    y=df["averagecoverage"].values

    # These are the centers of the bins
    x=list(np.arange(-extdto+(bs/2)-avg_centersize/2,(bs/2)-avg_centersize/2,bs))+central_bin_x_values+list(np.arange(avg_centersize/2+(bs/2),extdto+avg_centersize/2+(bs/2),bs))
    assert len(x)==len(y)

    labels=list(df.iloc[list(range(0,len(y),50)),:]["bin"].values)

    # Combined model consists of three gaussian models, and a constant model for the intercept
    comb_dipmodel=GaussianModel(prefix='G1_')+GaussianModel(prefix='G2_')+GaussianModel(prefix='G3_')+ConstantModel(prefix="Const_")
    # center of the Gaussian models must be zero (at the center of the core regions)
    comb_dipmodel.set_param_hint('G1_center',value=0,vary=False)
    comb_dipmodel.set_param_hint('G2_center',value=0,vary=False)
    comb_dipmodel.set_param_hint('G3_center',value=0,vary=False)
    
    # This part allows restrictions to the widths of the function. E.g. the average widths over all samples can be given as fixed widths here.
    comb_dipmodel.set_param_hint('G1_sigma',value=100,min=g1_min_sigma, max=g1_max_sigma)
    comb_dipmodel.set_param_hint('G2_sigma',value=2000,min=g2_min_sigma, max=g2_max_sigma)
    comb_dipmodel.set_param_hint('G3_sigma',value=7000,min=g3_min_sigma, max=g3_max_sigma)
    comb_dipmodel.set_param_hint('Const_c',value=0)
    comb_dipmodel_paramas=comb_dipmodel.make_params()

    comb_dipmodel=comb_dipmodel.fit(data=y,x=x,params=comb_dipmodel_paramas, method=method)

    comps = comb_dipmodel.eval_components()

    # Sort the gaussians such that the narrowest one is the first:
    sortedgaussians=[("G1_",comb_dipmodel.best_values["G1_sigma"]),("G2_",comb_dipmodel.best_values["G2_sigma"]),("G3_",comb_dipmodel.best_values["G3_sigma"])]
    sortedgaussians=sorted(sortedgaussians,key=lambda x: x[1])


    plt.plot(x,comb_dipmodel.best_fit, label="Combined model", linewidth=1.3)
    plt.plot(x, comps[sortedgaussians[0][0]],":",color="seagreen", label="First gaussian: "+r'$\sigma$'+"=%s"%(round(sortedgaussians[0][1],3)), alpha=0.5)
    plt.plot(x, comps[sortedgaussians[1][0]],"-.",color="purple", label="Second gaussian: "+r'$\sigma$'+"=%s"%(round(sortedgaussians[1][1],3)),alpha=0.5)
    plt.plot(x, comps[sortedgaussians[2][0]],"--",color="orange", label="Third gaussian: "+r'$\sigma$'+"=%s"%(round(sortedgaussians[2][1],3)),alpha=0.5)
    plt.plot(x, [comps["Const_"] for xvalue in x],"--",color="black", label="Intercept=%s"%(round(comb_dipmodel.best_values["Const_c"],3)),linewidth=0.5)

    plt.plot(x,y, "kx", alpha=0.2, label="Position-wise mean of bias corrected, RPGC normalized coverage")
    plt.title("%s-based fitting of 3 Gaussian Dips (Plus Intercept) to corrected coverage data for sample "%(method,)+samplename+"\n"+
              r'Constraints: center=0 ; %s $ \leq \sigma_{Gaussian 1} \leq $%s ; %s $\leq \sigma_{Gaussian 2} \leq $%s; '
              r'%s $\leq \sigma_{Gaussian 3} \leq $%s'%(g1_min_sigma,g1_max_sigma,g2_min_sigma,g2_max_sigma,g3_min_sigma,g3_max_sigma))

    total_depth=-(comb_dipmodel.params["G1_height"].value+comb_dipmodel.params["G2_height"].value+comb_dipmodel.params["G3_height"].value)

    plt.xlabel("Distance from center of Region of Interest [bp]")
    plt.ylabel("Normalized corrected coverage")
    plt.legend()
    fig=plt.gcf()
    fig.set_size_inches(16,9)
    fig.savefig(outdir+"/"+samplename+"_fitted_gaussians.png", )
    plt.close("all")
    
    intercept=comb_dipmodel.best_values["Const_c"]
    y_corr=[yval-intercept for yval in y]
    plt.plot(x,y_corr,alpha=0.7,color="navy")
    plt.ylabel("Bias corrected coverage, normalized by intercept")
    plt.xlabel("Distance from center of Region of Interest [bp]")
    plt.xticks([-15000,0,15000])
    plt.xlim(-15000,15000)
    fig=plt.gcf()
    fig.set_size_inches(16,9)
    fig.savefig(outdir+"/"+samplename+"_normalized_coverage.png", )
    plt.close()
    
    
    # Save the model parameters to a table
    for key, value in comb_dipmodel.params.valuesdict().items():
        if not "center" in key: # this is 0 anyway
            # this is required such that the narrow gaussian is always the G1:
            if "G" in key:
                for index,(name,sigma) in enumerate(sortedgaussians):
                    # key is ,e.g. G1_height. If G2 is the second-most narrow peak, rename G1_height to G2_height
                    if name.startswith(key[:2]):
                        sampledict["G"+str(index+1)+key[2:]]=value
                        
    sampledict["BIC"]=comb_dipmodel.bic     # Baysian Information Criterion
    sampledict["total_depth"]=total_depth
    sampledict["intercept"]=comb_dipmodel.best_values["Const_c"]

    # calculate area under the curve:
    # dont take x=0 as reference, but the intercept
    y_vals_for_AOC=[y-comps["Const_"] for y in comb_dipmodel.best_fit]
    y_vals_for_AOC_unfitted=[myy-comps["Const_"] for myy in y]

    y_vals_for_AOC_wo_smallest_gaussian_left=[y-comps["Const_"] for myx,y in zip(x,comb_dipmodel.best_fit) if (myx < -sampledict["G1_sigma"]) ]
    y_vals_for_AOC_wo_smallest_gaussian_right=[y-comps["Const_"] for myx,y in zip(x,comb_dipmodel.best_fit) if (myx > sampledict["G1_sigma"]) ]
    x_vals_for_AOC_wo_smallest_gaussian_left=[my_x for my_x  in x if (my_x < -sampledict["G1_sigma"]) ]
    x_vals_for_AOC_wo_smallest_gaussian_right=[my_x for my_x  in x if (my_x > sampledict["G1_sigma"]) ]

    sampledict["AOC_combined_model"]=np.trapz(x=x,y=y_vals_for_AOC)
    sampledict["AOC_comb_model_wo_smallest_gaussian"]=np.trapz(x=x_vals_for_AOC_wo_smallest_gaussian_left,y=y_vals_for_AOC_wo_smallest_gaussian_left)+\
                                                 np.trapz(x=x_vals_for_AOC_wo_smallest_gaussian_right,y=y_vals_for_AOC_wo_smallest_gaussian_right)
    sampledict["AOC_between_zero_and_bin_y_vals_fitted"]=sum(comb_dipmodel.best_fit)
  
    paramlist.append(sampledict)
param_df_all_samples=pd.DataFrame(paramlist)
param_df_all_samples.to_csv(outdir+"/parameter_summary_%s.csv"%(method,))
print("Fitting done - see the output table and .png in fitted_model/%s"%(outdir))
if print_sigmas:
    print(round(param_df_all_samples["G1_sigma"].median()),round(param_df_all_samples["G1_sigma"].median())+1,
          round(param_df_all_samples["G2_sigma"].median()),round(param_df_all_samples["G2_sigma"].median())+1,
          round(param_df_all_samples["G3_sigma"].median()),round(param_df_all_samples["G3_sigma"].median())+1)